/***********************************************************************
 * PROJECT NAME       : FZM
 * Component name     : SICP API
 * File name          : 
 * Author             : 
 * Date created       :
 * Date modified last :24.02.2014
 * ---------------------------------------------------------------------
 * Description        :
 *
 ***********************************************************************
 * Copyright (c) Nokia 2010 - 2012
 * All rights reserved.
 * Licensed material - property of Nokia
 ***********************************************************************
 * Revision history (latest on top)
 *
 * Date        | Who | Description
 * ---------------------------------------------------------------------
 * 24.02.2014  | Bincy Ashok | IPv6 Configuration changes for uplink
 ************************************************************************/
/** @file srio_iwf_uplink.c */

#ifndef __IUB_IP_UPLINK_SE_SRC_C__
#define __IUB_IP_UPLINK_SE_SRC_C__
#endif 

#include"iwf_table_creation.h"

#if 0
#define DBG_PRT(format, ...) printk("%s %d: " format, baseName(__FILE__), __LINE__, ##__VA_ARGS__);
#else
#define DBG_PRT(format, ...)
#endif

uint8_t dscp_pcp[DSCP_MAX];
uint8_t cp_dscp;
uint8_t mp_dscp;
uint32_t mp_IpAddr;
uint8_t vlanIds[MAX_VLANID];

extern _ipCounters ipCounters[VLANIDS_MAX];

extern sicp_ip_data_t   * lte_sicp_ip_table;
extern iwf_ip_addr_t_n  * ipList;
extern sicp_lte_ip_data_t  * sicp_lte_ip;
extern mbTacFpTmbData_t mbtacTmbCounterData;

extern unsigned int iwf_log_level;

bool gUplaneIpUpdated=false;
struct mutex gUPlaneIpMutexLock;

iwf_ip_addr_t mplaneIp;
#define IWF_LOG(log_level, ...)                      \
      do                                             \
      {                                              \
          if(log_level & iwf_log_level)              \
          {                                          \
               if(0 != printk_ratelimit())           \
               {                                     \
                   printk(__VA_ARGS__);              \
               }                                     \
          }                                          \
      }                                              \
while(0);                                      

#define IPV6OFFSET 2 
/*Define __IWF_UPLINK_DEBUG__ to enable uplink debug prints */

/*****************************************************************************
 * @Function Name        : set_sicp_teid_connection
 * Description           : To set teid connection
 * @Param [In]           : ccid_ul,sicp_teid_data
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t set_sicp_teid_connection (uint16_t ccid_ul, sicp_teid_data_t * sicp_teid_data)
{
    char  ip_str_src[40];
    char  ip_str_dst[40];
    sicp_ip_data_t * sicp_ip_entry = (sicp_ip_data_t *)( lte_sicp_ip_table + ccid_ul );

    if(NULL == sicp_ip_entry )
    {
        return ERR_INVALID_CONFIGURATION;  // value has already been configured
    }

    if ((sicp_ip_entry->uip_dst_index == 0) && 
            (is_ip_addr_null(&sicp_teid_data->uip_dst) == false) &&
            (is_ip_addr_null(&sicp_teid_data->uip_src) == false))

    {
        sicp_ip_entry->ip_src_index = 0;

        sicp_ip_entry->uip_dst_index = insert_ip(sicp_teid_data->uip_dst);
        sicp_ip_entry->uip_src_index = insert_ip(sicp_teid_data->uip_src);

        sicp_ip_entry->ip_ver                = sicp_teid_data->uip_dst.version;
        sicp_ip_entry->teid_ul               = sicp_teid_data->teid_ul;
        sicp_ip_entry->dst_port              = LTM_GTPU_PRT; // 2152
        sicp_ip_entry->src_port              = LTM_GTPU_PRT;
        sicp_ip_entry->dscp                  = sicp_teid_data->dscp;

        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
        { 
            if(0 != printk_ratelimit()) 
            {
                printk("IWF_UPLINK_DEBUG: Success  In %s() ip_dst: %s ip_src: %s\n teid_ul:%d dscp:%d ip_version in = %d ip version out %d\n", __FUNCTION__,
                        ((sicp_teid_data->uip_dst.version == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(sicp_teid_data->uip_dst.addr_u.ipv6Address, ip_str_dst)) :
                         (fzm_enet_inet_ntoa_r(sicp_teid_data->uip_dst.addr_u.ipv4Address, ip_str_dst))),
                        ((sicp_teid_data->uip_src.version == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(sicp_teid_data->uip_src.addr_u.ipv6Address, ip_str_src)) :
                         (fzm_enet_inet_ntoa_r(sicp_teid_data->uip_src.addr_u.ipv4Address, ip_str_src))),
                        sicp_teid_data->teid_ul,sicp_teid_data->dscp,sicp_teid_data->uip_dst.version, ipList[sicp_ip_entry->uip_dst_index].version);

            }
        }
        return VTC_OK;
    }

    if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level) 
    { 
        if(0 != printk_ratelimit()) 
        {
            printk("IWF_UPLINK_DEBUG: Error  In %s() ip_dst: %s ip_src: %s ccid_ul = %u ip version %d\n ", __FUNCTION__,
                    ((sicp_teid_data->uip_dst.version == IPv6)?
                     (fzm_enet_inet_ntoa_IPv6_r(sicp_teid_data->uip_dst.addr_u.ipv6Address, ip_str_dst)) :
                     (fzm_enet_inet_ntoa_r(sicp_teid_data->uip_dst.addr_u.ipv4Address, ip_str_dst))),
                    ((sicp_teid_data->uip_src.version == IPv6)?
                     (fzm_enet_inet_ntoa_IPv6_r(sicp_teid_data->uip_src.addr_u.ipv6Address, ip_str_src)) :
                         (fzm_enet_inet_ntoa_r(sicp_teid_data->uip_src.addr_u.ipv4Address, ip_str_src))), ccid_ul, sicp_teid_data->uip_src.version);
            }
        }

        return ERR_INVALID_CONFIGURATION;  // value has already been configured
    }

/*****************************************************************************
 * @Function Name        : ip_se_sicp_teid_set
 * Description           : To set teid connection
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_sicp_teid_set (sIpSicpApiMsg *msg)
{
    /* Up-link / egress processing */
    uint32_t           retVal    = VTC_ERR; // OK = 0 !!

    char               ip_str_src[40];
    char               ip_str_dst[40];

    ip_sicp_api_t    * cfg       = (ip_sicp_api_t *)     getCfgMsgPayload(msg);
    sicp_teid_data_t * conf_data = (sicp_teid_data_t *) &cfg->u.sicp_teid_data;

    if((is_ip_addr_null(&conf_data->uip_dst) == false) &&
      (is_ip_addr_null(&conf_data->uip_src) == false))
    {
        /* function call to write entry into the table */
        retVal = set_sicp_teid_connection(cfg->up.ccid_ul, conf_data);

        return retVal;
    } /* end of config_data_check */
    else
    {
        retVal = ERR_INVALID_CONFIGURATION;
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
        {
            if(0 != printk_ratelimit())
            {

                printk("IWF_UPLINK_DEBUG:  In %s()ip_dst: %s ip_src: %s\n", __FUNCTION__,
                        ((conf_data->uip_dst.version == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(conf_data->uip_dst.addr_u.ipv6Address, ip_str_dst)) :
                         (fzm_enet_inet_ntoa_r(conf_data->uip_dst.addr_u.ipv4Address, ip_str_dst))),
                        ((conf_data->uip_src.version == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(conf_data->uip_src.addr_u.ipv6Address, ip_str_src)) :
                         (fzm_enet_inet_ntoa_r(conf_data->uip_src.addr_u.ipv4Address, ip_str_src))));
            }
        }
        return retVal;
    }
}


/*****************************************************************************
 * @Function Name        : get_sicp_ip_connection
 * Description           : To get teid connection
 * @Param [In]           : ccid_ul,sicp_ip_table
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/
//NOTE: Debug prints in function will hog Debug log for UL traffic
// Please Note this before adding any debug print here
sicp_ip_data_t * get_sicp_ip_connection (uint16_t ccid_ul, sicp_ip_data_t *sicp_ip_table)
{
        sicp_ip_data_t * temp = (sicp_ip_data_t *)(sicp_ip_table + ccid_ul);

#if 0
        char               ip_str_dst[48];
        char               ip_str_src[48];

        printk("\nIWF_UPLINK_DEBUG:: In %s temp->ip_ver = %d teid_ul:%d dscp:%d ip_src:%s ip_dst=%s\n",
                __FUNCTION__,temp->ip_ver,temp->teid_ul,temp->dscp,
                (fzm_enet_inet_ntoa_IPv6_r(temp->uip_src.addr_u.ipv6Address, ip_str_src)),
                (fzm_enet_inet_ntoa_IPv6_r(temp->uip_dst.addr_u.ipv6Address, ip_str_dst)));
#endif

        if((is_ip_addr_null_n(&ipList[temp->uip_dst_index])) == true)
        {
            return NULL;
        }
        else
        {
            return temp;
        }
    }

uint32_t get_ip_se_lte_ip (uint8_t index)
{
    uint32_t ip=0;
    if (sicp_lte_ip[index].ip_ver == IPv4)
    {
        ip = sicp_lte_ip[index].uip_src.addr_u.ipv4Address.ipa_32;
        if(0 == ip)
        {
            return 0;
        }
        else
        {
            return ip;
        }
    }
    return 0;
}

iwf_ip_addr_t *get_up_ip_se_lte_ip (uint8_t index)
{
    return &sicp_lte_ip[index].uip_src;
}

/*****************************************************************************
 * @Function Name        : ip_se_sicp_teid_get
 * Description           : To get teid connection
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_sicp_teid_get (sIpSicpApiMsg *msg)
{
    /* Up-link / egress processing */
    uint32_t           retVal    = VTC_OK;
    char               ip_str_src[40];
    char               ip_str_dst[40];

    ip_sicp_api_t    * cfg       = (ip_sicp_api_t *) getCfgMsgPayload(msg);
    sicp_ip_data_t   * conf_data = get_sicp_ip_connection(cfg->up.ccid_ul, lte_sicp_ip_table);
    sicp_teid_data_t * ret_data  = (sicp_teid_data_t *) &cfg->u.sicp_teid_data;

    if (conf_data != NULL)
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
        {
            if(0 != printk_ratelimit())
            {

                printk("\nIWF_UPLINK_DEBUG:: In %s conf_data->ip_ver = %d d_ul:%d dscp:%hd ip_src:%s ip_dst=%s \n",
                        __FUNCTION__,ipList[conf_data->uip_dst_index].version,conf_data->teid_ul,conf_data->dscp,
                        (fzm_enet_inet_ntoa_IPv6_r(ipList[conf_data->uip_src_index].addr_u.ipv6Address, ip_str_src)),
                        (fzm_enet_inet_ntoa_IPv6_r(ipList[conf_data->uip_dst_index].addr_u.ipv6Address, ip_str_dst)));
            }
        }

        memcpy(&(ret_data->uip_dst), &ipList[conf_data->uip_dst_index], sizeof(iwf_ip_addr_t));
        memcpy(&(ret_data->uip_src), &ipList[conf_data->uip_src_index], sizeof(iwf_ip_addr_t));
        ret_data->teid_ul                       = conf_data->teid_ul;
        ret_data->dscp                          = conf_data->dscp;
        ret_data->ip_ver                        = ipList[conf_data->uip_dst_index].version ;

        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
        {
            if(0 != printk_ratelimit())
            {

                printk("IWF_UPLINK_DEBUG:: In %s()ip_dst: %s ip_src: %s\n teid_ul:%d dscp:%d ip_version = %d\n", __FUNCTION__,
                        ((ret_data->ip_ver == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(ret_data->uip_dst.addr_u.ipv6Address, ip_str_dst)) :
                         (fzm_enet_inet_ntoa_r(ret_data->uip_dst.addr_u.ipv4Address, ip_str_dst))),
                        ((ret_data->uip_src.version == IPv6)?
                         (fzm_enet_inet_ntoa_IPv6_r(ret_data->uip_src.addr_u.ipv6Address, ip_str_src)) :
                         (fzm_enet_inet_ntoa_r(ret_data->uip_src.addr_u.ipv4Address, ip_str_src))),
                        ret_data->teid_ul, ret_data->dscp,ret_data->ip_ver);
            }
        }
    }
    else
    {
        if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
        {
            if(0 != printk_ratelimit())
            {

                printk("IWF_UPLINK_DEBUG::Error !! In %s ip_se_sicp_teid_get conf_data is NULL\n",__FUNCTION__);
            }
        }
        retVal = VTC_ERR;
    }
    return retVal;
}

/*****************************************************************************
 * @Function Name        : del_sicp_teid_connection
 * Description           : To delete teid connection
 * @Param [In]           : ccid_ul
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t del_sicp_teid_connection (uint16_t ccid_ul)
{
        sicp_ip_data_t * sicp_ip_entry = (sicp_ip_data_t *)( lte_sicp_ip_table + ccid_ul);

        if(is_ip_addr_null_n(&ipList[sicp_ip_entry->uip_dst_index]) == false)
        {
            delete_ip(sicp_ip_entry->uip_dst_index);
            delete_ip(sicp_ip_entry->uip_src_index);
            memset((sicp_ip_data_t*)sicp_ip_entry, 0, sizeof(sicp_ip_data_t));
            return VTC_OK;
        }
        else
        {   
            if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
            {
                if(0 != printk_ratelimit())
                {

                    printk("IWF_UPLINK_DEBUG::Error !!In %s uip_dst is NULL ccid_ul %u\n",__FUNCTION__, ccid_ul);
                }
            }
            return VTC_ERR;  /* no value configured */
        }
}
/*****************************************************************************
 * @Function Name        : ip_se_sicp_teid_del
 * Description           : To delete teid connection
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_sicp_teid_del (sIpSicpApiMsg *msg)
{
    // Up-link / egress processing
    ip_sicp_api_t  * cfg = (ip_sicp_api_t *) getCfgMsgPayload(msg);

    uint32_t retVal = VTC_OK; // OK = 0 !!

    retVal = del_sicp_teid_connection(cfg->up.ccid_ul);

    return retVal; // OK = 0, ERR = 1
}

/*****************************************************************************
 * @Function Name        : mod_sicp_teid_dscp_val
 * Description           : To modify dscp value
 * @Param [In]           : ccid_ul,dscp
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t mod_sicp_teid_dscp_val (uint16_t ccid_ul, uint8_t dscp)
{
        uint32_t retVal = VTC_OK;
        sicp_ip_data_t * sicp_ip_entry = (sicp_ip_data_t *)(lte_sicp_ip_table + ccid_ul);

        if(is_ip_addr_null_n(&ipList[sicp_ip_entry->uip_dst_index]) == true)
        {
            if(IWF_LOG_LEVEL_TABLES_CONFIG & iwf_log_level)
            {
                if(0 != printk_ratelimit())
                {
                    printk("IWF_UPLINK_DEBUG::Error !!In %s uip_dst is NULL\n",__FUNCTION__);
                }
            }
            return VTC_ERR;
        }
        sicp_ip_entry->dscp            = dscp;
        return retVal;
}

/*****************************************************************************
 * @Function Name        : ip_se_sicp_teid_mod_dscp
 * Description           : To modify dscp value
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_sicp_teid_mod_dscp (sIpSicpApiMsg *msg)
{
    // Up-link / egress processing
    uint32_t           retVal    = VTC_ERR; // OK = 0 !!

    ip_sicp_api_t    * cfg       = (ip_sicp_api_t *)     getCfgMsgPayload(msg);
    sicp_teid_data_t * conf_data = (sicp_teid_data_t *) &cfg->u.sicp_teid_data;

 
    // function call to write entry into the table
    retVal = mod_sicp_teid_dscp_val(cfg->up.ccid_ul, conf_data->dscp);
    return retVal;
}


/*****************************************************************************
 * @Function Name        : set_ip_se_lte_ip
 * Description           : To set up ip
 * @Param [In]           : sicp_lte_ip_data,index
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t set_ip_se_lte_ip (sicp_lte_ip_data_t * sicp_lte_ip_data, uint8_t index)
{
     uint8_t i;
     char ip_str[40];

     if (LTE_MAX_IP_SRC_IDX <= index )
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG::Error In %s Invalid index %d\n",__FUNCTION__,index);
         return VTC_ERR;
     }

     for (i = 0; i < LTE_MAX_IP_SRC_IDX; i++)
     { // check existing IPs

         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: In %s() index = %d, i = %d, ip = %s line = %d\n",__FUNCTION__,index,i,((sicp_lte_ip_data->uip_src.version == IPv6) ?
                             (fzm_enet_inet_ntoa_IPv6_r(sicp_lte_ip_data->uip_src.addr_u.ipv6Address, ip_str)) :
                             (fzm_enet_inet_ntoa_r(sicp_lte_ip_data->uip_src.addr_u.ipv4Address, ip_str))),__LINE__);
         if (ip_is_equal(&sicp_lte_ip[i].uip_src, &sicp_lte_ip_data->uip_src))
         {
             IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: In %s: IP address %d %s already set!\n", __FUNCTION__, index,
                             ((sicp_lte_ip_data->uip_src.version == IPv6)?
                              (fzm_enet_inet_ntoa_IPv6_r(sicp_lte_ip_data->uip_src.addr_u.ipv6Address, ip_str)) :
                              (fzm_enet_inet_ntoa_r(sicp_lte_ip_data->uip_src.addr_u.ipv4Address, ip_str))));
             return VTC_ERR; // value already existing
         }
     }

     if(sicp_lte_ip_data->uip_src.version == IPv4)
     {
	sicp_lte_ip[index].uip_src.ip4_addr32 = sicp_lte_ip_data->uip_src.ip4_addr32;
        sicp_lte_ip[index].uip_src.version = IPv4;
	sicp_lte_ip[index].ip_ver = sicp_lte_ip_data->ip_ver;
     }
     else
     {
#if 0
        memcpy(&sicp_lte_ip[index + IPV6OFFSET].uip_src, &sicp_lte_ip_data->uip_src, sizeof(iwf_ip_addr_t));
	sicp_lte_ip[index + IPV6OFFSET].uip_src.version  = IPv6;
	sicp_lte_ip[index + IPV6OFFSET].ip_ver   = sicp_lte_ip_data->ip_ver;
#endif
        memcpy(&sicp_lte_ip[index].uip_src, &sicp_lte_ip_data->uip_src, sizeof(iwf_ip_addr_t));
	sicp_lte_ip[index].uip_src.version  = IPv6;
	sicp_lte_ip[index].ip_ver   = sicp_lte_ip_data->ip_ver;

     }


     /* set the flag to indicate that the u-plane ip is updated */
     if(!mutex_lock_interruptible(&gUPlaneIpMutexLock))
     {
         gUplaneIpUpdated = true;

         /* Unlock */
         mutex_unlock(&gUPlaneIpMutexLock);
     }
     return VTC_OK;
}

/*****************************************************************************
 * @Function Name        : ip_se_lte_ip_set
 * Description           : To set up ip
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_lte_ip_set (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;
     char ip_str[40];


     IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: In %s: %s \n", __FUNCTION__,
                     ((conf_data->uip_src.version == IPv6)?
                      (fzm_enet_inet_ntoa_IPv6_r (conf_data->uip_src.addr_u.ipv6Address, ip_str)) :
                      (fzm_enet_inet_ntoa_r(conf_data->uip_src.addr_u.ipv4Address, ip_str))));
     if (is_ip_addr_null(&conf_data->uip_src) == false)
     {
         set_ip_se_lte_ip(conf_data, conf_data->index) ;
         retVal = VTC_OK;
     }
     else 
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }

     return retVal;
}

/*****************************************************************************
 * @Function Name        : ip_se_lte_ip_get
 * Description           : To set up ip
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_lte_ip_get (sIpSicpApiMsg *msg)
{
    uint32_t retVal                = VTC_OK;
    ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
    sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;

    if ((conf_data->index < LTE_MAX_IP_SRC_IDX)
        && (get_up_ip_se_lte_ip(conf_data->index)))
    {
       memcpy(&cfg->u.sicp_lte_ip_data.uip_src, get_up_ip_se_lte_ip(conf_data->index), sizeof(iwf_ip_addr_t));
    }
    else
    {
        IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: Error In %s index %u\n",__FUNCTION__,conf_data->index);
        retVal = VTC_ERR;
    }
     return retVal;
}


/*****************************************************************************
 * @Function Name        : del_ip_se_lte_ip
 * Description           : To delete up ip
 * @Param [In]           : index
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

void del_ip_se_lte_ip (uint8_t index)
{
     memset(&sicp_lte_ip[index].uip_src, 0, sizeof(iwf_ip_addr_t));
}

/*****************************************************************************
 * @Function Name        : ip_se_lte_ip_del
 * Description           : To delete up ip
 * @Param [In]           : msg
 * Returns               : VTC_OK/VTC_ERR
 ****************************************************************************/

uint32_t ip_se_lte_ip_del (sIpSicpApiMsg *msg)
{
     uint32_t retVal                =  ERR_INVALID_INDEX_IN_LTE_IP_TABLE;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;
     uint32_t index = 0;

    if (is_ip_addr_null(&conf_data->uip_src) == false)
    {
        for (index = 0; index < LTE_MAX_IP_SRC_IDX; index++)
        {
           if (ip_is_equal(get_up_ip_se_lte_ip(index), &conf_data->uip_src))
           {
              del_ip_se_lte_ip(index);
              retVal = VTC_OK;
               break;
           }
        } 
     }
     else 
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: Error !! In %s uip_src is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
     return retVal;
}


/* ---------------------------------------------------------------------------- */

uint32_t ip_se_lte_DscpToPcp(sIpSicpApiMsg *msg)
{
    uint32_t retVal    = VTC_OK;
    memcpy(&dscp_pcp, &(msg->dscp_pcp), sizeof(dscp_pcp));
    return retVal;
}

uint32_t ip_se_lte_CplaneDscpToPcp(sIpSicpApiMsg *msg)
{
    uint32_t retVal    = VTC_OK;
    cp_dscp = msg->cp_dscp;
    return retVal;
}

uint32_t ip_se_lte_MplaneDscpToPcp(sIpSicpApiMsg *msg)
{
    uint32_t retVal    = VTC_OK;
    mp_dscp = msg->mp_dscp;
    mp_IpAddr = msg->mp_IpAddr;
    return retVal;
}


uint32_t ip_se_lte_cp_ip_set (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;
     uint32_t index = conf_data->index;
     char ip_str[40];

     IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: In %s: %s \n", __FUNCTION__,
                     ((conf_data->uip_src.version == IPv6)?
                      (fzm_enet_inet_ntoa_IPv6_r (conf_data->uip_src.addr_u.ipv6Address, ip_str)) :
                      (fzm_enet_inet_ntoa_r(conf_data->uip_src.addr_u.ipv4Address, ip_str))));

     if (is_ip_addr_null(&conf_data->uip_src) == false)
     {
         if(index < MAX_NO_CPLANE_IP_ADDR)
         {
            if(conf_data->uip_src.version == IPv4)
            { 
                 mbtacTmbCounterData.cplaneData[index].ip.ip4_addr32 = conf_data->uip_src.ip4_addr32;
                 mbtacTmbCounterData.cplaneData[index].ip.version = IPv4;
                 retVal = VTC_OK;
             }
             else
             {
                memcpy(&mbtacTmbCounterData.cplaneData[index + IPV6OFFSET ].ip,&conf_data->uip_src,sizeof(iwf_ip_addr_t));
                mbtacTmbCounterData.cplaneData[index + IPV6OFFSET ].ip.version   = conf_data->uip_src.version;
                retVal = VTC_OK;
             }
         }

     }
     else
     {
            IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: IP Address NULL \n");
            retVal = ERR_INVALID_CONFIGURATION;
     }
    
     return retVal;
}

uint32_t ip_se_lte_sp_ip_set (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;

     if (conf_data->uip_src.ip4_addr32 != 0 )
     {
         mbtacTmbCounterData.splaneData.ip.ip4_addr32 = conf_data->uip_src.ip4_addr32;
         mbtacTmbCounterData.splaneData.ip.version = conf_data->uip_src.version; 
         retVal = VTC_OK;
     }
     else
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
     return retVal;
}

uint32_t ip_se_lte_mp_ip_set (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;

     if (conf_data->uip_src.ip4_addr32 != 0 )
     {
         mplaneIp.ip4_addr32 = conf_data->uip_src.ip4_addr32;
         mplaneIp.version = conf_data->uip_src.version; 
         retVal = VTC_OK; 
     }
     else
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
     IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG ::%s() IP [0x%x]\n",__FUNCTION__, mplaneIp.ip4_addr32);
     return retVal;
}

uint32_t ip_se_lte_mp_ip_get (sIpSicpApiMsg *msg)
{
    return 0;
}

uint32_t ip_se_lte_mp_ip_del (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;

     if (conf_data->uip_src.ip4_addr32 != 0 )
     {
         if ( conf_data->uip_src.ip4_addr32 == mplaneIp.ip4_addr32)
         {
             memset(&mplaneIp, 0, sizeof(iwf_ip_addr_t));
             retVal = VTC_OK;
         }
     }
     else
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
    return 0;
}


//C-Plane IP
uint32_t ip_se_lte_cp_ip_get (sIpSicpApiMsg *msg)
{   
    return 0;
}   
uint32_t ip_se_lte_cp_ip_del (sIpSicpApiMsg *msg)
{    
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;
     uint32_t index=0 ;

     if (is_ip_addr_null(&conf_data->uip_src) == false)
     {
         for (index = 0; index < MAX_NO_CPLANE_IP_ADDR; index++)
         {
            if(conf_data->uip_src.version == IPv4)
            {
                if ( conf_data->uip_src.ip4_addr32 == mbtacTmbCounterData.cplaneData[index].ip.ip4_addr32)
                {
                    memset(&mbtacTmbCounterData.cplaneData[index].ip, 0, sizeof(iwf_ip_addr_t));
                    retVal = VTC_OK;
                    break;
                }
             }
             else
             {
                if(!memcmp(&conf_data->uip_src,&mbtacTmbCounterData.cplaneData[index].ip,sizeof(iwf_ip_addr_t)))
                {
                    memset(&mbtacTmbCounterData.cplaneData[index].ip, 0, sizeof(iwf_ip_addr_t));
                    retVal = VTC_OK;
                    break;
                }
             }
         }
     }
     else
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG,"IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
    return retVal;
}

//S-Plane IP
uint32_t ip_se_lte_sp_ip_get (sIpSicpApiMsg *msg)
{
    return 0;
}
uint32_t ip_se_lte_sp_ip_del (sIpSicpApiMsg *msg)
{
     uint32_t retVal                = ERR_INVALID_INDEX_IN_LTE_IP_TABLE ;
     ip_sicp_api_t      * cfg       = (ip_sicp_api_t *)getCfgMsgPayload(msg);
     sicp_lte_ip_data_t * conf_data = (sicp_lte_ip_data_t *) &cfg->u.sicp_lte_ip_data;

     if (conf_data->uip_src.ip4_addr32 != 0 )
     {
         if ( conf_data->uip_src.ip4_addr32 == mbtacTmbCounterData.splaneData.ip.ip4_addr32)
         {
             memset(&mbtacTmbCounterData.splaneData.ip, 0, sizeof(iwf_ip_addr_t));
             retVal = VTC_OK;
         }
     }
     else
     {
         IWF_LOG(IWF_LOG_LEVEL_TABLES_CONFIG, "IWF_UPLINK_DEBUG :: Error In %s IP is NULL\n",__FUNCTION__);
         retVal = ERR_INVALID_CONFIGURATION;
     }
    return 0;
}

uint32_t ip_se_uplink_proc_dump (struct seq_file *m)
{
    uint32_t i;
    uint32_t num_connections = 0;
    sicp_ip_data_t *conf_data;
    int retVal = 0;
    char ip_str[40];

    if (NULL != lte_sicp_ip_table)
    {
        /* only if valid pointer, else we would get a seg fault */
        seq_printf(m,"\nLTE IWF Uplink table:\n");
        seq_printf(m,"================================================================================================\n");
        seq_printf(m,"Configured LTE connections for UL (SICP->IP) IWF:\n");
        seq_printf(m,"================================================================================================\n");
        seq_printf(m,"CCID   IP_Dest                                   TEID_UL   UDP_DST  UDP_SRC   IP_IDX   DSCP\n");
        seq_printf(m,"=====  =======================================  =========  =======  =======   =======  ====== \n");

        for (i = 0; i < LTE_WCDMA_CCID_MAX_VALUE; i++)
        {
            conf_data = get_sicp_ip_connection(i, lte_sicp_ip_table);
            //Print only valid entries (entry !=0)
            if (conf_data != NULL)
            {
                if (ipList[conf_data->uip_dst_index].version == IPv4)
                {
                    iwf_ip_addr_t ip;
                    ip.addr_u.ipv4Address.ipa_32 = htonl(ipList[conf_data->uip_dst_index].addr_u.ipv4Address.ipa_32);
                    seq_printf(m,"0x%04x    %-39s  0x%08x  %-6u    %-6u    %-3u    0x%04x\n",
                            i,
                            fzm_enet_inet_ntoa_r(ip.addr_u.ipv4Address, ip_str),
                            //fzm_enet_inet_ntoa_r(conf_data->uip_dst.addr_u.ipv4Address, ip_str),
                            conf_data->teid_ul,
                            conf_data->dst_port,
                            conf_data->src_port,
                            conf_data->ip_src_index,
                            conf_data->dscp);
                    num_connections++;
                }
                else if (ipList[conf_data->uip_dst_index].version == IPv6)
                {
                    iwf_ip_addr_t ip;
                    ip.addr_u.ipv6Address.ipa_32[0] = htonl(ipList[conf_data->uip_dst_index].addr_u.ipv6Address.ipa_32[0]);
                    ip.addr_u.ipv6Address.ipa_32[1] = htonl(ipList[conf_data->uip_dst_index].addr_u.ipv6Address.ipa_32[1]); 
                    ip.addr_u.ipv6Address.ipa_32[2] = htonl(ipList[conf_data->uip_dst_index].addr_u.ipv6Address.ipa_32[2]);
                    ip.addr_u.ipv6Address.ipa_32[3] = htonl(ipList[conf_data->uip_dst_index].addr_u.ipv6Address.ipa_32[3]); 

                    seq_printf(m,"0x%04x    %-39s  0x%08x  %-6u    %-6u    %-3u    0x%04x\n",
                            i,
                            fzm_enet_inet_ntoa_IPv6_r(ip.addr_u.ipv6Address, ip_str),
                            //fzm_enet_inet_ntoa_IPv6_r(conf_data->uip_dst.addr_u.ipv6Address, ip_str),
                            conf_data->teid_ul,
                            conf_data->dst_port,
                            conf_data->src_port,
                            conf_data->ip_src_index,
                            conf_data->dscp);
                }
                else
                {
                    seq_printf(m,"IP version not set !!!\n");
                }
            }
        }
        seq_printf(m,"=====\n");
        seq_printf(m,"Total number of connections in LTE IWF Uplink table: %u\n", num_connections);
        seq_printf(m,"------------------------------------------------------------------------\n");
    }
    else
    {
        seq_printf(m,"LTE IWF Uplink table pointer = NULL\n");
        retVal = 1;
    }

    return retVal;
}

uint32_t ip_se_lte_up_proc_dump (struct seq_file *m)
{
    uint32_t index;
    iwf_ip_addr_t *uip;
    int retVal = 0;
    char ip_str[40];

    /* only if valid pointer, else we would get a seg fault */
    seq_printf(m,"\n\n");
    seq_printf(m,"================================================================================================\n");
    seq_printf(m,"Configured LTE U-Plane IPs in IWF:\n");
    seq_printf(m,"================================================================================================\n");
    seq_printf(m,"U-Plane IP\n");
    seq_printf(m,"=======================================\n");

    for (index = 0; index < LTE_MAX_IP_SRC_IDX; index++)
    {
        uip = get_up_ip_se_lte_ip(index);
        if (uip != NULL)
        {
            if(uip->version == IPv4)
            {
                iwf_ip_addr_t ip;

                ip.addr_u.ipv4Address.ipa_32 = htonl(uip->addr_u.ipv4Address.ipa_32);

                seq_printf(m,"%-39s\n", fzm_enet_inet_ntoa_r(ip.addr_u.ipv4Address, ip_str));
            }
            else if(uip->version == IPv6)
            {
                iwf_ip_addr_t ip;

                ip.addr_u.ipv6Address.ipa_32[0] = htonl(uip->addr_u.ipv6Address.ipa_32[0]);
                ip.addr_u.ipv6Address.ipa_32[1] = htonl(uip->addr_u.ipv6Address.ipa_32[1]); 
                ip.addr_u.ipv6Address.ipa_32[2] = htonl(uip->addr_u.ipv6Address.ipa_32[2]);
                ip.addr_u.ipv6Address.ipa_32[3] = htonl(uip->addr_u.ipv6Address.ipa_32[3]); 

                seq_printf(m,"%-39s\n", fzm_enet_inet_ntoa_IPv6_r(ip.addr_u.ipv6Address, ip_str));
            }
        }
    }

    return retVal;
}
uint32_t ip_se_VlanidToGroupIdMap(sIpSicpApiMsg *msg) 
{ 
       uint32_t retVal    = VTC_OK; 
       int i; 
       uint16_t vlanMap[VLANIDS_MAX]; 
    
       memcpy(&vlanMap, &(msg->vlanMap), sizeof(vlanMap)); 
       memset(vlanIds,0,sizeof(vlanIds)); 
       for(i=0;i<VLANIDS_MAX;i++) 
       { 
           if(ipCounters[i].vlanId != vlanMap[i]) 
              memset(&ipCounters[i],0,sizeof(_ipCounters)); 

          if(vlanMap[i]) 
          { 
             printk("vlan id passed %d\n",vlanMap[i]); 
   	     vlanIds[vlanMap[i]] = i+1; 
          } 
       } 
       return retVal; 
}    

